#ifndef _MAINWIDGET_CPP_
#define _MAINWIDGET_CPP_

#include <fluid.h>
#include <QGridLayout>
#include <QDockWidget>
#include <QTabWidget>
#include <QStringList>
#include <QMenu>
#include <QMenuBar>
#include <QAction>
#include <QApplication>
#include <QDesktopWidget>
#include <QFileDialog>
#include <QSplashScreen>

namespace fluid {

	const string MainWidget::VIEWOPTION_STANDARD = "standard map";
	const string MainWidget::VIEWOPTION_VORTICITY = "vorticity map";

	void MainWidget::updateMaxGlyphs() {
		fluidListener->setMaxGlyphs((unsigned int)maxGlyphs->getValue());
	}

	void MainWidget::updateMaxStateSwitchTime() {
		Glyph::setMaxStateSwitchTime(maxStateSwitchTime->getValue());
	}

	void MainWidget::updateMaxRespawnDistance() {
		Glyph::setMaxRespawnDistance(maxRespawnDistance->getValue());
	}

	void MainWidget::updateMaxKillDistance() {
		Glyph::setMaxKillDistance(maxKillDistance->getValue());
	}

	void MainWidget::updateViewOptions() {
		if(viewOptions->currentText().compare( tr(VIEWOPTION_STANDARD.c_str()) ) == 0) {
			fluidListener->setView(VIEW_STANDARD);
		} else if(viewOptions->currentText().compare( tr(VIEWOPTION_VORTICITY.c_str()) ) == 0) {
			fluidListener->setView(VIEW_VORTICITY);
		}
	}

	void MainWidget::loadSolid() {
		string solidpath = QFileDialog::getOpenFileName(this,tr("Open a map for the solids"),tr(""),tr("*.png *.bmp *.jpg")).toStdString();
		try {
			fluidListener->setSolidMap(solidpath);
		} catch(...) {
			WarningWidget *warning = new WarningWidget(tr("Loading Error"),tr("Error: could not load image!"),this);
			warning->setModal(true);
			warning->show();
			return;
		}
	}

	void MainWidget::loadBackground() {
		string backgroundpath = QFileDialog::getOpenFileName(this,tr("Open a map for the background"),tr(""),tr("*.png *.bmp *.jpg")).toStdString();
		try {
			fluidListener->setBackgroundMap(backgroundpath);
		} catch(...) {
			WarningWidget *warning = new WarningWidget(tr("Loading Error"),tr("Error: could not load image!"),this);
			warning->setModal(true);
			warning->show();
			return;
		}
	}

	void MainWidget::showInfo() {
		QSplashScreen *splashScreen = new QSplashScreen(QPixmap("../data/icons/splashscreen.png"));
		splashScreen->show();
	}

	MainWidget::MainWidget(): QMainWindow() {
		QWidget *central_Widget = new QWidget();
		setCentralWidget(central_Widget);
		QGridLayout *layout = new QGridLayout();
		central_Widget->setLayout(layout);

		FluidLayout *fWidget = new FluidLayout();
		QGridLayout *fLayout = new QGridLayout();
		fWidget->setLayout(fLayout);

		fluidListener = new FluidListener();
		SXWidget *renderWidget = new SXWidget();

		QSizePolicy policy(QSizePolicy::Expanding, QSizePolicy::Expanding);
		policy.setHeightForWidth(true);
		fWidget->setSizePolicy(policy);
		renderWidget->renderPeriodically(0.01f);
		QObject::connect(renderWidget,SIGNAL(finishedRendering()),renderWidget,SLOT(close()));
		renderWidget->addRenderListener(*fluidListener);
		fLayout->addWidget(renderWidget);
		layout->addWidget(fWidget,1,1);

		//add dock widget
		QDockWidget *parameters = new QDockWidget("Parameters");
		QWidget* titleWidget = new QWidget(this);
		parameters->setTitleBarWidget( titleWidget );
		parameters->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
		QTabWidget *parametersTabs = new QTabWidget();
		parametersTabs->setTabPosition(QTabWidget::East);
		parameters->setWidget(parametersTabs);
		addDockWidget(Qt::RightDockWidgetArea,parameters);

		//parameters tab
		QWidget *parametersTab_top = new QWidget();
		QGridLayout *parametersTab_intermediateLayout = new QGridLayout();
		parametersTab_top->setLayout(parametersTab_intermediateLayout);
		QWidget *parametersTabWidget = new QWidget();
		QGridLayout *parametersTab = new QGridLayout();
		parametersTabWidget->setLayout(parametersTab);
		parametersTab_intermediateLayout->addWidget(parametersTabWidget,0,0,Qt::AlignTop);
		parametersTabWidget->setLayout(parametersTab);
		parametersTabs->addTab(parametersTab_top,tr("Parameters"));

		maxGlyphs = new IntSlider("number of glyphs", 160, 1, 160);
		maxGlyphs->setMinimumWidth(200);
		parametersTab->addWidget(maxGlyphs,0,0);
		QObject::connect(maxGlyphs,SIGNAL(valueChanged(int)),this,SLOT(updateMaxGlyphs()));

		maxStateSwitchTime = new FloatSlider("kill/respawn time", 0.25f, 0.0f, 2.0f);
		maxStateSwitchTime->setMinimumWidth(200);
		parametersTab->addWidget(maxStateSwitchTime,1,0);
		QObject::connect(maxStateSwitchTime,SIGNAL(valueChanged(double)),this,SLOT(updateMaxStateSwitchTime()));

		maxRespawnDistance = new FloatSlider("respawn distance", 0.05f, 0.0f, 0.25f);
		maxRespawnDistance->setMinimumWidth(200);
		parametersTab->addWidget(maxRespawnDistance,2,0);
		QObject::connect(maxRespawnDistance,SIGNAL(valueChanged(double)),this,SLOT(updateMaxRespawnDistance()));

		maxKillDistance = new FloatSlider("death distance", 0.05f, 0.0f, 0.25f);
		maxKillDistance->setMinimumWidth(200);
		parametersTab->addWidget(maxKillDistance,3,0);
		QObject::connect(maxKillDistance,SIGNAL(valueChanged(double)),this,SLOT(updateMaxKillDistance()));
		
		QWidget *viewOptionsWidget = new QWidget();
		QGridLayout *viewOptionsLayout = new QGridLayout();
		viewOptionsWidget->setLayout(viewOptionsLayout);
		QStringList viewList;
		viewList << tr(VIEWOPTION_STANDARD.c_str()) << tr(VIEWOPTION_VORTICITY.c_str());
		viewOptions = new QComboBox();
		viewOptions->addItems(viewList);
		parametersTab->addWidget(viewOptionsWidget,4,0);
		viewOptionsLayout->addWidget(new QLabel("map"),0,0);
		viewOptionsLayout->addWidget(viewOptions,0,1);
		QObject::connect(viewOptions,SIGNAL(currentIndexChanged(int)),this,SLOT(updateViewOptions()));

		QAction *openSolidAction = new QAction(QIcon("../data/icons/open.png"),"Load solid map from image", this);
		QObject::connect(openSolidAction,SIGNAL(triggered()),this,SLOT(loadSolid()));

		QAction *openBackgroundAction = new QAction(QIcon("../data/icons/open.png"), "Load background image", this);
		QObject::connect(openBackgroundAction,SIGNAL(triggered()),this,SLOT(loadBackground()));

		QAction *exitAction = new QAction("Exit", this);
		QObject::connect(exitAction,SIGNAL(triggered()),this,SLOT(close()));

		QAction *infoAction = new QAction(QIcon("../data/icons/fluid.png"), "About fluid with arrows", this);
		QObject::connect(infoAction,SIGNAL(triggered()),this,SLOT(showInfo()));

		QMenu *fileMenu = menuBar()->addMenu("File");
		fileMenu->addAction(openSolidAction);
		fileMenu->addAction(openBackgroundAction);
		fileMenu->addSeparator();
		fileMenu->addAction(exitAction);

		//creation menu
		QMenu *terrainCreationMenu = menuBar()->addMenu("Help");
		terrainCreationMenu->addAction(infoAction);

		setFixedSize(sizeHint());

		QDesktopWidget *desktop = QApplication::desktop();
		int screenWidth = desktop->width();
		int screenHeight = desktop->height();
		int x = (screenWidth - width()) / 2;
		int y = (screenHeight - height()) / 2;
		setGeometry(x, y, width(), height());
	}

	MainWidget::~MainWidget() {
	}

}

#endif