#ifndef _FLUID_STREAMLETGRID_CPP_
#define _FLUID_STREAMLETGRID_CPP_

#include <fluid.h>

namespace fluid {

	StreamletGrid::StreamletGrid(unsigned int cells_x, unsigned int cells_y) {
		this->cells_x = cells_x;
		this->cells_y = cells_y;
		grid = new list<GridElement *> *[cells_x];
		for(unsigned int x=0 ; x<cells_x ; x++) {
			grid[x] = new list<GridElement *>[cells_y];
		}
	}

	StreamletGrid::~StreamletGrid() {
		clear();
	}

	void StreamletGrid::put(Glyph *glyph) {
		for(unsigned int i=0 ; i<glyph->getGroupSize() ; i = i+3) {
			//create gridelement, and put it into
			//the list of elements
			GridElement *elem = new GridElement();
			elem->glyph = glyph;
			elem->position = (*glyph)[i];
			elem->length = glyph->length();
			elem->position[2] = 0;
			elem->position[3] = 1;
			elements.push_back(elem);

			//calculate grid indices
			Matrix t = Matrix().translate(Vector(0.5f,0.5f,0.0f)) * Matrix().scale(Vector(0.5f));
			Vector gridvec = t * elem->position;
			int i_x = (int)floor(((float)cells_x)*gridvec[0]);
			int i_y = (int)floor(((float)cells_y)*gridvec[1]);
			i_x = min(max(i_x , 0) , (int)cells_x - 1);
			i_y = min(max(i_y , 0) , (int)cells_y - 1);
			elem->x = (unsigned int)i_x;
			elem->y = (unsigned int)i_y;
			
			//put gridelement into gridcell (i_x,i_y)
			//and its eight neighbours
			for(int x=-1 ; x<=1 ; x++) {
				for(int y=-1 ; y<=1 ; y++) {
					int ix = i_x + x;
					int iy = i_y + y;
					if(ix < 0 || ix >= (int)cells_x || iy < 0 || iy >= (int)cells_y) {
						continue;
					}
					grid[ix][iy].push_back(elem);
				}
			}
		}
		
	}

	void StreamletGrid::clear() {
		for(list<GridElement *>::iterator iter = elements.begin() ; iter != elements.end() ; iter++) {
			delete (*iter);
		}
		elements.clear();
		for(unsigned int x=0 ; x<cells_x ; x++) {
			for(unsigned int y=0 ; y<cells_y ; y++) {
				grid[x][y].clear();
			}
		}
	}

	void StreamletGrid::collide() {
		for(list<GridElement *>::iterator iter = elements.begin() ; iter != elements.end() ; iter++) {
			GridElement *e = (*iter);
			if(e->glyph->getState() != STATE_ALIVE) {
				continue;
			}
			list<GridElement *> &neighbourhood = grid[e->x][e->y];
			for(list<GridElement *>::iterator i2 = neighbourhood.begin() ; i2 != neighbourhood.end() ; i2++) {
				GridElement *e2 = (*i2);
				if(e->glyph == e2->glyph || e2->glyph->getState() != STATE_ALIVE) {
					continue;
				}
				float maxd = max( e->glyph->killDistance() , e2->glyph->killDistance() );
				if(maxd > e->position.distance(e2->position)) {
					if(e->length > e2->length) {
						e2->glyph->kill();
					} else {
						e->glyph->kill();
					}
				}
			}
		}
	}

	bool StreamletGrid::respawnCollide(Glyph *g) {
		Vector p = (*g)[0];
		p[2] = 0;
		p[3] = 1;

		//calculate grid indices
		Matrix t = Matrix().translate(Vector(0.5f,0.5f,0.0f)) * Matrix().scale(Vector(0.5f));
		Vector gridvec = t * p;
		int i_x = (int)floor(((float)cells_x)*gridvec[0]);
		int i_y = (int)floor(((float)cells_y)*gridvec[1]);
		i_x = min(max(i_x , 0) , (int)cells_x - 1);
		i_y = min(max(i_y , 0) , (int)cells_y - 1);

		//do collition detection in neighbouring cells
		for(int x=-1 ; x<=1 ; x++) {
			for(int y=-1 ; y<=1 ; y++) {
				int ix = i_x + x;
				int iy = i_y + y;
				if(ix < 0 || ix >= (int)cells_x || iy < 0 || iy >= (int)cells_y) {
					continue;
				}

				list<GridElement *> &neighbourhood = grid[ix][iy];
				for(list<GridElement *>::iterator i2 = neighbourhood.begin() ; i2 != neighbourhood.end() ; i2++) {
					GridElement *e = (*i2);
					if(e->glyph->respawnDistance() > e->position.distance(p)) {
						return true;
					}
				}
			}
		}
	
		return false;
	}

}

#endif